package volkov.starling.filter
{
	import starling.filters.FragmentFilter;
	import starling.rendering.FilterEffect;
	
	
	/**
	 * Оверлей.
	 * Аналогичен обычному overlay эффекту в фотошопе.
	 * Starling 2.0, AGAL 1.
	 * @author Roman A.V.
	 */
	public class OverlayFilter extends FragmentFilter {
		
		// Приват
		private var _color:uint;
		
		
		/**
		 * Создать фильтр оверлея.
		 * @param	color Цвет в формате RGB.
		 */
		public function OverlayFilter(color:uint = 0x000000) {
			super();
			this.color = color;
		}
		
		
		// ПРИВАТ
		override protected function createEffect():FilterEffect {
			return new OverlayEffect(_color);
		}
		private function get overlayEffect():OverlayEffect {
			return this.effect as OverlayEffect;
		}
		
		
		// ГЕТТЕР-СЕТТЕРЫ
		/**
		 * Цвет в формате RGB.
		 */
		public function get color():uint {
			return _color;
		}
		public function set color(value:uint):void {
			if (value === _color)
				return;
			
			_color = value;
			
			overlayEffect.setColor(value);
			setRequiresRedraw();
		}
	}
}
import flash.display3D.Context3D;
import flash.display3D.Context3DProgramType;
import starling.rendering.FilterEffect;
import starling.rendering.Program;
import starling.utils.Color;
class OverlayEffect extends FilterEffect {
	
	static private const CONSTANTS:Vector.<Number> = new <Number>[0.5, 1, 3, 0];
	
	// Приват
	private var _colorVec:Vector.<Number>; // Цвет шейдера.
	
	/// Создать эффект оверлея.
	public function OverlayEffect(color:uint) {
		super();
		
		_colorVec = new Vector.<Number>(4, true);
		setColor(color);
	}
	
	
	// ПАБЛИК
	public function setColor(color:uint):void {
		_colorVec[0] = Color.getRed(color) / 255.0;		// RED
		_colorVec[1] = Color.getGreen(color) / 255.0;	// GREEN
		_colorVec[2] = Color.getBlue(color) / 255.0;	// BLUE
		_colorVec[3] = 1.0;								// ALPHA
	}
	override protected function createProgram():Program {
		var vertexShader:String		= FilterEffect.STD_VERTEX_SHADER;
		var fragmentShader:String	= 	tex("ft0", "v0", 0, texture) +
										
										// В ft0.x получаем чернобелый цвет пикселя:
										"add ft0.x, ft0.x, ft0.y \n" + 
										"add ft0.x, ft0.x, ft0.z \n" + 
										"div ft0.x, ft0.x, fc1.z \n" +
										
										// Находим количество цвета в пикселе, где 1 - чисто серый:
										"div ft0.x, ft0.x, fc1.x \n" +
										"mul ft1.xyzw, ft0.xxxw, fc0.xyzw \n" +		// <-- Цвет пиксела без добавления белого.
										
										// Добавляем белого: (Если количества цвета больше 1)
										"sub ft0.y, ft0.x, fc1.y \n" +
										"max ft0.y, ft0.y, fc1.w \n" + 
										"add oc, ft1.xyzw, ft0.yyyw \n";
		
		return Program.fromSource(vertexShader, fragmentShader, 1);
	}
	override protected function beforeDraw(context:Context3D):void {
		super.beforeDraw(context);
		
		context.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 0, _colorVec);
		context.setProgramConstantsFromVector(Context3DProgramType.FRAGMENT, 1, CONSTANTS);
	}
}